@@ -70,6 +70,7 @@ typedef struct {
 	GIOChannel *channel;		/* our GIO to the daemon */
 	GHashTable *handlers;		/* hash of msg -> handler */
 	gboolean authorized;
+	gboolean active_scanning;
 
 	EggTrayIcon *tray_icon;
 	GtkWidget *button;
@@ -90,6 +91,9 @@ typedef struct {
 
 static NetApplet *netapplet;
 
+static guint update_id = 0;
+static gboolean window_shown = FALSE;
+
 static void populate_popup_menu (void);
 
 static void
@@ -598,11 +602,24 @@ show_key_dialog (const char *essid)
 		const char *key;
 
 		key = gtk_entry_get_text (GTK_ENTRY (entry_key));
+		key = netcommon_verify_string (key);
+		if (!key) {
+			GtkWidget *error_dialog;
 
-		netapplet_set_essid (old_essid, key);
-
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox_keyring)))
+			error_dialog = gtk_message_dialog_new_with_markup (
+				NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+				_("<span weight=\"bold\" size=\"larger\">"
+				  "Invalid Encryption Key: "
+				  "</span>\n\n"
+				  "Key contains illegal characters!"));
+			gtk_dialog_run (GTK_DIALOG (error_dialog));
+			gtk_widget_destroy (error_dialog);
+		} else {
+			netapplet_set_essid (old_essid, key);
+			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
+					(checkbox_keyring)))
 			keyring_save (old_essid, key);
+		}
 	}
 
 	g_free (old_essid);
@@ -654,6 +671,8 @@ queue_get (gpointer user_data)
 
 	if (netapplet->default_keyring != NULL)
 		get_item (essid);
+	else
+		show_key_dialog (essid);
 }
 
 static void
@@ -708,12 +727,43 @@ other_essid_cb (GtkMenuItem *mi G_GNUC_UNUSED, gpointer user_data G_GNUC_UNUSED)
 		essid = gtk_entry_get_text (GTK_ENTRY (entry_essid));
 		key = gtk_entry_get_text (GTK_ENTRY (entry_key));
 
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox_keyring)))
-			keyring_save (essid, key);
+		essid = netcommon_verify_string (essid);
+		if (!essid || *essid == '\0') {
+			GtkWidget *error_dialog;
 
+			error_dialog = gtk_message_dialog_new_with_markup (
+				NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+				_("<span weight=\"bold\" size=\"larger\">"
+				  "Invalid ESSID: "
+				  "</span>\n\n"
+				  "ESSID is blank or "
+				  "contains illegal characters!"));
+			gtk_dialog_run (GTK_DIALOG (error_dialog));
+			gtk_widget_destroy (error_dialog);
+			goto out;
+		}
+
+		key = netcommon_verify_string (key);
+		if (!key) {
+			GtkWidget *error_dialog;
+
+			error_dialog = gtk_message_dialog_new_with_markup (
+				NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+				_("<span weight=\"bold\" size=\"larger\">"
+				"Invalid Encryption Key:</span>\n\n"
+				"Key contains illegal characters!"));
+			gtk_dialog_run (GTK_DIALOG (error_dialog));
+			gtk_widget_destroy (error_dialog);
+			goto out;
+		}
+
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
+					(checkbox_keyring)))
+			keyring_save (essid, key);
 		netapplet_set_essid (essid, key);
 	}
 
+out:
 	gtk_widget_destroy (dialog);
 	g_object_unref (xml);
 }
@@ -732,45 +782,49 @@ get_label (GtkWidget *info_dialog, GladeXML *xml, const char *name)
 	return label;
 }
 
-static void
-show_info_cb (GtkMenuItem *mi G_GNUC_UNUSED, gpointer user_data G_GNUC_UNUSED)
+static gboolean
+update_info(GladeXML *xml)
 {
-	static GtkWidget *info_dialog = NULL;
+	GtkWidget *info_dialog = NULL;
 	char *addr = NULL, *mask = NULL, *broadcast = NULL;
 	char *dest = NULL, *mac = NULL;
 	GtkWidget *label;
-	GladeXML *xml = NULL;
 	Connection *conn;
 	struct ifreq ifr;
 	int fd, flags;
+	gboolean ret_val = TRUE;
+
+	info_dialog = glade_xml_get_widget (xml, "dialog-information");
 
 	conn = netapplet->active;
 	if (!conn) {
 		GtkWidget *error_dialog;
 
 		error_dialog = gtk_message_dialog_new_with_markup (
-				NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+				window_shown ? GTK_WINDOW (info_dialog) : NULL,
+				0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
 				_("<span weight=\"bold\" size=\"larger\">"
 				  "Error displaying connection information: "
 				  "</span>\n\n"
 				  "No active connection!"));
 		gtk_dialog_run (GTK_DIALOG (error_dialog));
 		gtk_widget_destroy (error_dialog);
-		return;
+		return FALSE;
 	}
 
 	fd = socket (AF_INET, SOCK_DGRAM, 0);
 	if (fd < 0) {
 		GtkWidget *error_dialog;
 
-		error_dialog = gtk_message_dialog_new_with_markup (NULL, 0,
-				GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+		error_dialog = gtk_message_dialog_new_with_markup (
+				window_shown ? GTK_WINDOW (info_dialog) : NULL,
+				0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
 				_("<span weight=\"bold\" size=\"larger\">"
 				  "Error displaying connection information: "
 				"</span>\n\nUnable to open socket!"));
 		gtk_dialog_run (GTK_DIALOG (error_dialog));
 		gtk_widget_destroy (error_dialog);
-		return;
+		return FALSE;
 	}
 
 	ifr.ifr_addr.sa_family = AF_INET;
@@ -784,14 +838,16 @@ show_info_cb (GtkMenuItem *mi G_GNUC_UNUSED, gpointer user_data G_GNUC_UNUSED)
 	if (ioctl (fd, SIOCGIFFLAGS, &ifr) < 0) {
 		GtkWidget *error_dialog;
 
-		error_dialog = gtk_message_dialog_new_with_markup (NULL, 0,
-				GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+		error_dialog = gtk_message_dialog_new_with_markup (
+				window_shown ? GTK_WINDOW (info_dialog) : NULL,
+				0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
 				_("<span weight=\"bold\" size=\"larger\">"
 				  "Error displaying information: "
 				  "</span>\n\n"
 				  "SIOCGIFFLAGS failed on socket!"));
 		gtk_dialog_run (GTK_DIALOG (error_dialog));
 		gtk_widget_destroy (error_dialog);
+		ret_val = FALSE;
 		goto out;
 	}
 	flags = ifr.ifr_flags;
@@ -821,11 +877,6 @@ show_info_cb (GtkMenuItem *mi G_GNUC_UNUSED, gpointer user_data G_GNUC_UNUSED)
 		       (unsigned char) ifr.ifr_hwaddr.sa_data[4],
 		       (unsigned char) ifr.ifr_hwaddr.sa_data[5]);
 
-	if (!info_dialog) {
-		xml = glade_xml_new (GLADE_APPLET, "dialog-information", NULL);
-		info_dialog = glade_xml_get_widget (xml, "dialog-information");
-	}
-
 	label = get_label (info_dialog, xml, "label-interface");
 	gtk_label_set_text (GTK_LABEL (label), conn->interface);
 
@@ -859,14 +910,6 @@ show_info_cb (GtkMenuItem *mi G_GNUC_UNUSED, gpointer user_data G_GNUC_UNUSED)
 	label = get_label (info_dialog, xml, "label-hardware-address");
 	gtk_label_set_text (GTK_LABEL (label), mac);
 
-	if (xml != NULL) {
-		gtk_dialog_run (GTK_DIALOG (info_dialog));
-		gtk_widget_destroy (info_dialog);
-		info_dialog = NULL;
-		g_object_unref (xml);
-	} else
-		gtk_window_present (GTK_WINDOW (info_dialog));
-
 out:
 	close (fd);
 	g_free (addr);
@@ -874,6 +917,60 @@ out:
 	g_free (mask);
 	g_free (dest);
 	g_free (mac);
+
+	return ret_val;
+}
+
+static gboolean
+update_cb (gpointer user_data)
+{
+    GladeXML *xml = user_data;
+    if (update_info (xml)) {
+	return TRUE;
+    } else {
+	update_id = 0;
+	return FALSE;
+    }
+}
+
+static void
+show_info_cb (GtkMenuItem *mi G_GNUC_UNUSED, gpointer user_data G_GNUC_UNUSED)
+{
+	static GladeXML *xml = NULL;
+	GtkWidget *info_dialog = NULL;
+
+	if (!xml) {
+		xml = glade_xml_new (GLADE_APPLET, "dialog-information", NULL);
+	}
+
+	info_dialog = glade_xml_get_widget (xml, "dialog-information");
+
+	if (window_shown) {
+		if (!update_info(xml))
+			return;
+
+		gtk_window_present (GTK_WINDOW (info_dialog));
+		if (update_id == 0) {
+			update_id = g_timeout_add (1000, update_cb, xml);
+		}
+		return;
+	}
+
+	if (!update_info(xml))
+	    return;
+
+	update_id = g_timeout_add (1000, update_cb, xml);
+
+	window_shown = TRUE;
+	gtk_window_present (GTK_WINDOW (info_dialog));
+	gtk_dialog_run (GTK_DIALOG (info_dialog));
+	gtk_widget_hide (GTK_WIDGET (info_dialog));
+	window_shown = FALSE;
+
+	if (update_id) {
+	    g_source_remove (update_id);
+	}
+	update_id = 0;
 }
 
 static void
@@ -1202,19 +1299,41 @@ menu_position_func (GtkMenu *menu G_GNUC_UNUSED, int *x, int *y,
 	*push_in = TRUE;
 }
 
+static void
+active_scan_on (void)
+{
+	netapplet->active_scanning = TRUE;
+	if (!netapplet->active)
+		return;
+	if (g_str_has_prefix (netapplet->active->interface, "ath")) {
+		netapplet_get_accesspoints (netapplet->active->interface);
+		netapplet_get_wireless (netapplet->active->interface);
+	}
+}
+
+static void
+active_scan_off (void)
+{
+	netapplet->active_scanning = FALSE;
+}
+
 static void
 button_toggled_cb (GtkToggleButton *button G_GNUC_UNUSED,
 		   gpointer user_data G_GNUC_UNUSED)
 {
-	gtk_menu_popup (GTK_MENU (netapplet->popup_menu), NULL, NULL,
-			menu_position_func, NULL, 0,
-			gtk_get_current_event_time ());
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (netapplet->button))) {
+		active_scan_on ();
+		gtk_menu_popup (GTK_MENU (netapplet->popup_menu), NULL, NULL,
+				menu_position_func, NULL, 0,
+				gtk_get_current_event_time ());
+	}
 }
 
 static void
 menu_deactivate_cb (GtkMenuShell *menu_shell G_GNUC_UNUSED,
 		    gpointer user_data G_GNUC_UNUSED)
 {
+	active_scan_off ();
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (netapplet->button),
 				      FALSE);
 	gtk_menu_popdown (GTK_MENU (netapplet->popup_menu));
@@ -1520,8 +1639,11 @@ netapplet_handle_active (GIOChannel *channel G_GNUC_UNUSED, char **args)
 	}
 
 	if (g_str_has_prefix (netapplet->active->type, "Wireless")) {
-		netapplet_get_accesspoints (netapplet->active->interface);
-		netapplet_get_wireless (netapplet->active->interface);
+		if (netapplet->active_scanning || changed ||
+		    !g_str_has_prefix (netapplet->active->interface, "ath")) {
+			netapplet_get_accesspoints (netapplet->active->interface);
+			netapplet_get_wireless (netapplet->active->interface);
+		}
 	} else if (netapplet->accesspoints) {
 		/* remove no-longer-valid wireless information */
 		netapplet_free_accesspoints ();
@@ -1578,6 +1700,8 @@ netapplet_new (void)
 	applet = g_new0 (NetApplet, 1);
 	applet->tray_icon = egg_tray_icon_new ("applet");
 
+	applet->active_scanning = FALSE;
+
 	applet->button = gtk_toggle_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (applet->button),
 			       GTK_RELIEF_NONE);
