@@ -32,6 +32,9 @@
 #define IFUP_BIN		"/sbin/ifup"
 #define IFDOWN_BIN		"/sbin/ifdown"
 #define ROUTE_CMD               "/sbin/route"
+#define FORWARD_CMD             SBINDIR "/netapplet-forward"
+#define STARTFWD_CMD            SBINDIR "/netapplet-startfwd"
+#define STOPFWD_CMD             SBINDIR "/netapplet-stopfwd"
 #define MAX_QUAL_OVERRIDE	90	/* for driver's with a broken max */
 
 #define ROUTE_ARGV              { ROUTE_CMD, "-n", NULL }
@@ -162,29 +165,301 @@ get_interfaces (void)
 	return iface_list;
 }
 
+static char *
+get_config_path (const char *interface)
+{
+	const char *argv[3];
+	char *getcfg;
+	char *cfg;
+	char *p;
+	GError *err = NULL;	
+	char *retval;
+
+	argv[0] = GETCFG_CMD;
+	argv[1] = interface;
+	argv[2] = NULL;
+
+	retval = NULL;
+
+	/* get the configuration information for this interface */
+	if (!g_spawn_sync (NULL, (char **) argv, NULL, 0, NULL, NULL,
+			   &getcfg, NULL, NULL, &err)) {
+			g_warning ("Unable to execute "GETCFG_CMD": %s",
+				   err->message);
+			g_error_free (err);
+			return NULL;
+	}
+
+	/* get the ifcfg name for this interface */
+	cfg = strstr (getcfg, "HWD_CONFIG_0");
+	if (!cfg) {
+		g_warning ("Unable to determine configuration file for "
+			   "interface %s", interface);
+		goto out_free_getcfg;
+	}
+	cfg = strstr (cfg, "=");
+	if (!cfg) {
+		g_warning ("Unable to determine configuration file for "
+			   "interface %s", interface);
+		goto out_free_getcfg;
+	}
+	cfg++;
+	p = strstr (cfg, ";");
+	if (!p) {
+		g_warning ("Unable to determine configuration file for "
+			   "interface %s", interface);
+		goto out_free_getcfg;
+	}
+	p[0] = '\0';
+	cfg = g_strdup_printf (NETWORK_SYSCONF_PATH"/"IFCFG_PREFIX"%s", cfg);
+
+	retval = cfg;
+
+ out_free_getcfg:
+	g_free (getcfg);
+	return retval;
+}
+
+static char *
+escape_essid (const char *essid)
+{
+	char **tokens;
+	char *escaped;
+
+	tokens = g_strsplit (essid, "'", 0);
+	escaped = g_strjoinv ("'\\''", tokens);
+	g_strfreev (tokens);
+
+	return escaped;
+}
+
+/*
+ * Rewrites the /etc/sysconfig/network file for the specified interface.
+ *
+ * If any field is specified as NULL, then we simply skip that field,
+ * and don't overwrite any existing values.
+ *
+ * interface: interface config to modify (e.g. "eth0")
+ * bootproto: "static" for static IP, "dhcp" for dhcp
+ * ipaddr: IP address ("18.238.0.1"), or NULL to skip
+ * netmask: netmask ("255.255.255.0"), or NULL to skip
+ * wireless_mode: Must be either 'Managed' 'Ad-Hoc' or 'Master'
+ * essid: essid or NULL to skip
+ * key: Must be either a wireless encryption key or "" if no key
+ */
+static gboolean
+modify_interface_config (const char *interface, const char *bootproto,
+			 const char *ipaddr, const char *netmask,
+			 const char *wireless_mode,
+			 const char *wireless_essid,
+			 const char *wireless_key)
+{
+	GString *output;
+	char *data = NULL, *cfg;
+	GIOChannel *file;
+	GError *err = NULL;
+	gboolean retval = FALSE;
+
+	cfg = get_config_path (interface);
+	if (cfg == NULL)
+		return FALSE;
+
+	output = g_string_sized_new (512);  /* start big to prevent resizing */
+
+	file = g_io_channel_new_file (cfg, "r", &err);
+	if (!file) {
+		g_warning ("Failed to open %s: %s\n", cfg, err->message);
+		g_error_free (err);
+		return FALSE;
+	}
+
+	/* remove lines that we are changing */
+	while (g_io_channel_read_line (file, &data, NULL, NULL, &err) ==
+			G_IO_STATUS_NORMAL) {
+		if (!(g_str_has_prefix (data, "WIRELESS_ESSID=") && wireless_essid != NULL) &&
+		    !(g_str_has_prefix (data, "WIRELESS_KEY_0=") && wireless_key != NULL) &&
+		    !(g_str_has_prefix (data, "WIRELESS_MODE") && wireless_mode != NULL) &&
+		    !(g_str_has_prefix (data, "IPADDR=") && ipaddr != NULL) &&
+		    !(g_str_has_prefix (data, "NETMASK=") && netmask != NULL) &&
+		    !(g_str_has_prefix (data, "BOOTPROTO=") && bootproto != NULL) &&
+
+		    /* we always reset these */
+		    !g_str_has_prefix (data, "WIRELESS_NWID=''") &&
+		    !g_str_has_prefix (data, "WIRELESS_KEY=''") &&
+		    !g_str_has_prefix (data, "WIRELESS_KEY_LENGTH=") &&
+		    !g_str_has_prefix (data, "WIRELESS_AP") &&
+		    !g_str_has_prefix (data, "WIRELESS_BITRATE") &&
+		    !g_str_has_prefix (data, "WIRELESS_CHANNEL") &&
+		    !g_str_has_prefix (data, "WIRELESS_FREQUENCY") &&
+		    !g_str_has_prefix (data, "WIRELESS_DEFAULT_KEY="))
+			g_string_append (output, data);
+
+		g_free (data);
+	}
+
+	if (err) {
+		g_warning ("Failed to read %s: %s\n", cfg, err->message);
+		g_error_free (err);
+		goto out;
+	}
+
+	/* write out new network configuration lines */
+	if (bootproto != NULL)
+		g_string_append_printf (output, "BOOTPROTO='%s'\n", bootproto);
+
+	if (ipaddr != NULL && strlen (ipaddr) > 0)
+		g_string_append_printf (output, "IPADDR='%s'\n", ipaddr);
+
+	if (netmask != NULL && strlen (netmask) > 0)
+		g_string_append_printf (output, "NETMASK='%s'\n", netmask);
+
+	/* write out wireless configuration lines */
+	if (! strcmp (get_network_type (interface), TYPE_WIRELESS) &&
+	    (wireless_essid != NULL || wireless_mode != NULL || wireless_key != NULL)) {
+
+		if (wireless_essid != NULL) {
+			char *escaped_essid;
+			escaped_essid = escape_essid (wireless_essid);
+			g_string_append_printf (output, "WIRELESS_ESSID='%s'\n",
+						escaped_essid);
+			g_free (escaped_essid);
+		}
+
+		if (wireless_key != NULL && strlen (wireless_key) > 0)
+			g_string_append_printf (output, "WIRELESS_KEY_0='%s'\n", wireless_key);
+
+		/* return the other lines to a nice sane default */
+		if (! strcmp (wireless_mode, ""))
+			wireless_mode = "Managed";
+
+		g_string_append_printf (output, "WIRELESS_NWID=''\n");
+		g_string_append_printf (output, "WIRELESS_KEY=''\n");
+		g_string_append_printf (output, "WIRELESS_DEFAULT_KEY='0'\n");
+		g_string_append_printf (output, "WIRELESS_KEY_LENGTH='128'\n");
+		g_string_append_printf (output, "WIRELESS_AP=''\n");
+		g_string_append_printf (output, "WIRELESS_BITRATE='auto'\n");
+		g_string_append_printf (output, "WIRELESS_CHANNEL=''\n");
+		g_string_append_printf (output, "WIRELESS_MODE='%s'\n", wireless_mode);
+		g_string_append_printf (output, "WIRELESS_FREQUENCY=''");
+	}
+
+	/* close, truncate, and reopen the file for writing */
+	g_io_channel_unref (file);
+	file = g_io_channel_new_file (cfg, "w", &err);
+	if (!file) {
+		g_warning ("Failed to open %s: %s\n", cfg, err->message);
+		g_error_free (err);
+		goto out;
+	}
+
+	/* write out the new file in one swoop */
+	if (g_io_channel_write_chars (file, output->str, -1, NULL, &err) !=
+			G_IO_STATUS_NORMAL) {
+		g_warning ("Failed to write to %s: %s\n", cfg, err->message);
+		g_error_free (err);
+		goto out;
+	}
+
+	g_io_channel_flush (file, NULL);
+	retval = TRUE;
+
+out:
+	g_string_free (output, TRUE);
+	g_io_channel_unref (file);
+	g_free (cfg);
+
+	return retval;
+}
+
+static gboolean
+ifup (const char *interface)
+{
+	const char *argv[3];
+	GError *err = NULL;
+	int ret;
+
+	argv[0] = IFUP_BIN;
+	argv[1] = interface;
+	argv[2] = NULL;
+
+	if (!g_spawn_sync (NULL, (char **) argv, NULL,
+			   0, NULL, NULL, NULL, NULL, &ret,
+			   &err)) {
+		g_warning ("Unable to exec " IFUP_BIN ": %s",
+			   err->message);
+		g_error_free (err);
+	}
+
+	if (WEXITSTATUS (ret) == 0)
+		return TRUE;
+
+	return FALSE;
+}
+
+static void
+ifdown (const char *interface)
+{
+	const char *argv[3];
+	GError *err = NULL;
+
+	argv[0] = IFDOWN_BIN;
+	argv[1] = interface;
+	argv[2] = NULL;
+
+	if (!g_spawn_sync (NULL, (char **) argv, NULL,
+			   0, NULL, NULL, NULL, NULL, NULL,
+			   &err)) {
+		g_warning ("Unable to exec " IFDOWN_BIN ": %s",
+			   err->message);
+		g_error_free (err);
+	}
+}
+
+#if 0
+static void
+setup_forward (const char *from, const char *to)
+{
+	char *cmd;
+
+	/* FIXME: exploitable? */
+	cmd = g_strdup_printf ("%s %s %s", FORWARD_CMD, from, to);
+	g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
+}
+#endif
+
+/*
+ * Sanitize the interface.  We cannot trust the networking shell
+ * scripts, which have escaping problems out the wazoo.
+ */
+static const char *
+verify_interface (const char *interface)
+{
+	GSList *iface_list, *iter;
+
+	if (!interface)
+		return NULL;
+
+	iface_list = get_interfaces ();
+	if (!iface_list)
+		return NULL;
+
+	for (iter = iface_list; iter != NULL; iter = iter->next) {
+		if (strcmp (iter->data, interface) == 0)
+			return interface;
+	}
+
+	return NULL;
+}
+
 static void
 netdaemon_disconnect_all (void)
 {
 	GSList *iface_list, *iter;
-	GError *err = NULL;
 
 	iface_list = get_interfaces ();
 
-	for (iter = iface_list; iter != NULL; iter = iter->next) {
-		const char *argv[3], *interface = iter->data;
-
-		argv[0] = IFDOWN_BIN;
-		argv[1] = interface;
-		argv[2] = NULL;
-
-		if (!g_spawn_sync (NULL, (char **) argv, NULL,
-				   0, NULL, NULL, NULL, NULL, NULL,
-				   &err)) {
-			g_warning ("Unable to exec " IFDOWN_BIN ": %s",
-				   err->message);
-			g_error_free (err);
-		}
-	}
+	for (iter = iface_list; iter != NULL; iter = iter->next)
+		ifdown (iter->data);
 
 	g_free (active_iface);
 	active_iface = NULL;
@@ -205,12 +480,14 @@ netdaemon_do_disconnect (GIOChannel *channel, char **args G_GNUC_UNUSED)
 static void
 netdaemon_do_change_active (GIOChannel *channel G_GNUC_UNUSED, char **args)
 {
-	const char *argv[3];
-	GError *err = NULL;
-	int ret;
+	const char *interface;
 
-	if (!args[1])
+	/* Is this interface valid ? */
+	interface = verify_interface (args[1]);
+	if (!interface) {
+		g_warning ("invalid interface: %s\n", args[1]);
 		return;
+	}
 
 	/*
 	 * To get around various network script bugs, and to have the concept
@@ -219,22 +496,20 @@ netdaemon_do_change_active (GIOChannel *channel G_GNUC_UNUSED, char **args)
 	 */
 	netdaemon_disconnect_all ();
 
-	argv[0] = IFUP_BIN;
-	argv[1] = args[1];
-	argv[2] = NULL;
+	/*
+	 * In case the wireless interface was left in Master mode with
+	 * a static IP from sharing earlier, reset it to Managed mode with
+	 * a dynamic IP now.
+	 */
+	if (!strcmp (get_network_type (interface), TYPE_WIRELESS))
+		modify_interface_config (interface, "dhcp", NULL, NULL,
+					 "Managed", NULL, NULL);
 
-	if (!g_spawn_sync (NULL, (char **) argv, NULL,
-			   0, NULL, NULL, NULL, NULL, &ret,
-			   &err)) {
-		g_warning ("Unable to exec " IFUP_BIN ": %s",
-			   err->message);
-		g_error_free (err);
-	}
-
-	if (WEXITSTATUS (ret) == 0) {
-		netcommon_send_message (channel, "active", args[1], NULL);
+	/* Bring the interface up */
+	if (ifup (interface)) {
+		netcommon_send_message (channel, "active", interface, NULL);
 		g_free (active_iface);
-		active_iface = g_strdup (args[1]);
+		active_iface = g_strdup (interface);
 	}
 
 	/* 
@@ -261,7 +536,7 @@ netdaemon_do_list_interfaces (GIOChannel *channel G_GNUC_UNUSED,
 		const char *type;
 
 		if (active_iface != NULL &&
-		    strcmp (active_iface, interface) == 0)
+				strcmp (active_iface, interface) == 0)
 			found_active = TRUE;
 
 		type = get_network_type (interface);
@@ -346,6 +621,74 @@ out:
 	g_free (output);
 }
 
+#if 0
+/*
+ * turn on IP forwarding in the kernel
+ *
+ * args[0]: 'start_forward' (ignored)
+ */
+static void
+netdaemon_do_start_forward (GIOChannel *channel G_GNUC_UNUSED, char **args G_GNUC_UNUSED)
+{
+	g_spawn_command_line_sync (STARTFWD_CMD, NULL, NULL, NULL, NULL);
+}
+
+/*
+ * turn off IP forwarding in the kernel, and clear iptables
+ *
+ * args[0]: 'start_forward' (ignored)
+ */
+static void
+netdaemon_do_stop_forward (GIOChannel *channel G_GNUC_UNUSED, char **args G_GNUC_UNUSED)
+{
+	g_spawn_command_line_sync (STOPFWD_CMD, NULL, NULL, NULL, NULL);
+}
+
+/*
+ * handler to share one interface to another interface
+ *
+ * args[0]: 'share_interface' (ignored)
+ * args[1]: from_interface
+ * args[2]: to_interface
+ */
+static void
+netdaemon_do_share_interface (GIOChannel *channel G_GNUC_UNUSED, char **args)
+{
+	char *from_ifc = args[1];
+	char *to_ifc = args[2];
+
+	printf ("netdaemon_do_share_interface\n");
+
+	ifup (to_ifc);
+
+	setup_forward (from_ifc, to_ifc);
+}
+
+/*
+ * handler to setup a wireless interface as a wireless access point
+ *
+ * args[0]: 'setup_wap' (ignored)
+ * args[1]: wireless interface
+ * args[2]: new IP address to setup
+ * args[3]: new netmask to setup
+ * args[4]: new ESSID to setup
+ * args[5]: new wireless key to setup
+ */
+static void
+netdaemon_do_setup_wap (GIOChannel *channel  G_GNUC_UNUSED, char **args)
+{
+	const char *interface = args[1];
+	const char *ipaddr    = args[2];
+	const char *netmask   = args[3];
+	const char *essid     = args[4];
+	const char *key       = args[5];
+
+	printf ("netdaemon_do_setup_wap\n");
+	
+	modify_interface_config (interface, "static", ipaddr, netmask, "Master", essid, key);
+}
+#endif
+
 static APInfo *
 parse_scanning_event (struct iw_event *event, APInfo *old_ap)
 {
@@ -360,8 +703,10 @@ parse_scanning_event (struct iw_event *event, APInfo *old_ap)
 
 	switch (event->cmd) {
 	case SIOCGIWESSID:
-		memcpy (ap->essid, event->u.essid.pointer,
-			IW_ESSID_MAX_SIZE + 1);
+		if (event->u.essid.pointer) {
+			memcpy (ap->essid, event->u.essid.pointer,
+				IW_ESSID_MAX_SIZE + 1);
+		}
 		ap->essid[event->u.essid.length] = '\0';
 		break;
 	case IWEVQUAL:
@@ -564,10 +909,16 @@ netdaemon_get_link_quality (int fd, const char *interface)
 static void
 netdaemon_do_get_accesspoints (GIOChannel *channel, char **args)
 {
-	const char *interface = args[1];
+	const char *interface;
 	struct iwreq wrq;
 	int fd;
 
+	interface = verify_interface (args[1]);
+	if (!interface) {
+		g_warning ("invalid interface: %s\n", args[1]);
+		return;
+	}
+
 	fd = iw_sockets_open ();
 	if (fd < 0)
 		return;
@@ -602,10 +953,10 @@ netdaemon_do_get_accesspoints (GIOChannel *channel, char **args)
 		strncpy (wrq.ifr_name, interface, IFNAMSIZ);
 		if (ioctl (fd, SIOCGIWAP, &wrq) >= 0) {
 			char ap_addr[128];
-			
+
 			memset (ap_addr, 0, 128);
 			iw_pr_ether (ap_addr, wrq.u.ap_addr.sa_data);
-			
+
 			if (!is_valid_ap_address (ap_addr)) {
 				error = TRUE;
 				goto out;
@@ -640,7 +991,7 @@ netdaemon_do_get_accesspoints (GIOChannel *channel, char **args)
 
 		quality = netdaemon_get_link_quality (fd, interface);
 
-	out:
+out:
 		if (error)
 			msg = g_strdup_printf ("accesspoints %s", interface);
 		else {
@@ -662,19 +1013,6 @@ netdaemon_do_get_accesspoints (GIOChannel *channel, char **args)
 	}
 }
 
-static char *
-escape_essid (const char *essid)
-{
-	char **tokens;
-	char *escaped;
-
-	tokens = g_strsplit (essid, "'", 0);
-	escaped = g_strjoinv ("'\\''", tokens);
-	g_strfreev (tokens);
-
-	return escaped;
-}
-
 /*
  * handler to change the current wireless interface's ESSID
  *
@@ -686,136 +1024,38 @@ escape_essid (const char *essid)
 static void
 netdaemon_do_change_essid (GIOChannel *channel, char **args)
 {
-	GString *output;
-	const char *argv[3];
-	char *data = NULL, *getcfg, *p, *cfg;
-	GIOChannel *file;
-	GError *err = NULL;
-	char *escaped_essid;
+	const char *interface, *essid, *key;
 
-	argv[0] = GETCFG_CMD;
-	argv[1] = args[1];
-	argv[2] = NULL;
-
-	/* get the configuration information for this interface */
-	if (!g_spawn_sync (NULL, (char **) argv, NULL, 0, NULL, NULL,
-			   &getcfg, NULL, NULL, &err)) {
-			g_warning ("Unable to execute "GETCFG_CMD": %s",
-				   err->message);
-			g_error_free (err);
-			return;
+	interface = verify_interface (args[1]);
+	if (!interface) {
+		g_warning ("invalid interface: %s\n", args[1]);
+		return;
+	}
+	essid = netcommon_verify_string (args[2]);
+	if (!essid) {
+		g_warning ("invalid essid: %s\n", args[2]);
+		return;
+	}
+	key = netcommon_verify_string (args[3]);
+	if (!key) {
+		g_warning ("invalid key: %s\n", args[3]);
+		return;
 	}
 
-	/* get the ifcfg name for this interface */
-	cfg = strstr (getcfg, "HWD_CONFIG_0");
-	if (!cfg) {
-		g_warning ("Unable to determine configuration file for "
-			   "interface %s", args[1]);
-		goto out_free_getcfg;
-	}
-	cfg = strstr (cfg, "=");
-	if (!cfg) {
-		g_warning ("Unable to determine configuration file for "
-			   "interface %s", args[1]);
-		goto out_free_getcfg;
-	}
-	cfg++;
-	p = strstr (cfg, ";");
-	if (!p) {
-		g_warning ("Unable to determine configuration file for "
-			   "interface %s", args[1]);
-		goto out_free_getcfg;
-	}
-	p[0] = '\0';
-	cfg = g_strdup_printf (NETWORK_SYSCONF_PATH"/"IFCFG_PREFIX"%s", cfg);
-
-	output = g_string_sized_new (512);  /* start big to prevent resizing */
-
-	file = g_io_channel_new_file (cfg, "r", &err);
-	if (!file) {
-		g_warning ("Failed to open %s: %s\n", cfg, err->message);
-		g_error_free (err);
-		goto out_free_getcfg;
-	}
-
-	/* remove the offending lines */
-	while (g_io_channel_read_line (file, &data, NULL, NULL, &err) ==
-			G_IO_STATUS_NORMAL) {
-		if (!g_str_has_prefix (data, "WIRELESS_ESSID=") &&
-		    !g_str_has_prefix (data, "WIRELESS_NWID=''") &&
-		    !g_str_has_prefix (data, "WIRELESS_KEY=''") &&
-		    !g_str_has_prefix (data, "WIRELESS_KEY_0=") &&
-		    !g_str_has_prefix (data, "WIRELESS_KEY_LENGTH=") &&
-		    !g_str_has_prefix (data, "WIRELESS_AP") &&
-		    !g_str_has_prefix (data, "WIRELESS_BITRATE") &&
-		    !g_str_has_prefix (data, "WIRELESS_CHANNEL") &&
-		    !g_str_has_prefix (data, "WIRELESS_MODE") &&
-		    !g_str_has_prefix (data, "WIRELESS_FREQUENCY") &&
-		    !g_str_has_prefix (data, "WIRELESS_DEFAULT_KEY="))
-			g_string_append (output, data);
-
-		g_free (data);
-	}
-
-	if (err) {
-		g_warning ("Failed to read %s: %s\n", cfg, err->message);
-		g_error_free (err);
-		goto out;
-	}
-
-	/* add back the updated lines */
-	escaped_essid = escape_essid (args[2]);
-	g_string_append_printf (output, "WIRELESS_ESSID='%s'\n",
-				escaped_essid);
-	g_free (escaped_essid);
-
-	g_string_append_printf (output, "WIRELESS_KEY_0='%s'\n", args[3]);
-
-	/* return the other lines to a nice sane default */
-	g_string_append_printf (output, "WIRELESS_NWID=''\n");
-	g_string_append_printf (output, "WIRELESS_KEY=''\n");
-	g_string_append_printf (output, "WIRELESS_DEFAULT_KEY='0'\n");
-	g_string_append_printf (output, "WIRELESS_KEY_LENGTH='128'\n");
-	g_string_append_printf (output, "WIRELESS_AP=''\n");
-	g_string_append_printf (output, "WIRELESS_BITRATE='auto'\n");
-	g_string_append_printf (output, "WIRELESS_CHANNEL=''\n");
-	g_string_append_printf (output, "WIRELESS_MODE='Managed'\n");
-	g_string_append_printf (output, "WIRELESS_FREQUENCY=''");
-
-	/* close, truncate, and reopen the file for writing */
-	g_io_channel_unref (file);
-	file = g_io_channel_new_file (cfg, "w", &err);
-	if (!file) {
-		g_warning ("Failed to open %s: %s\n", cfg, err->message);
-		g_error_free (err);
-		goto out;
-	}
-
-	/* write out the new file in one swoop */
-	if (g_io_channel_write_chars (file, output->str, -1, NULL, &err) !=
-			G_IO_STATUS_NORMAL) {
-		g_warning ("Failed to write to %s: %s\n", cfg, err->message);
-		g_error_free (err);
-		goto out;
-	}
-
-	g_io_channel_flush (file, NULL);
-
-	netdaemon_do_change_active (channel, args);
-
-out:
-	g_string_free (output, TRUE);
-	g_io_channel_unref (file);
-	g_free (cfg);
-
-out_free_getcfg:
-	g_free (getcfg);
+	if (modify_interface_config (interface,	/* interface */
+				     "dhcp",	/* bootproto */
+				     NULL,	/* ip address */
+				     NULL,	/* netmask */
+				     "Managed",	/* Wireless mode */
+				     essid,	/* essid */
+				     key))	/* key */
+		netdaemon_do_change_active (channel, args);
 }
 
 static void
 netdaemon_do_get_wireless (GIOChannel *channel, char **args)
 {
-	const char *interface = args[1];
+	const char *interface;
 	int skfd;
 	struct iwreq wrq;
 	char essid[IW_ESSID_MAX_SIZE + 1];
@@ -824,6 +1064,12 @@ netdaemon_do_get_wireless (GIOChannel *channel, char **args)
 	char *escaped_essid, *msg;
 	float quality;
 
+	interface = verify_interface (args[1]);
+	if (!interface) {
+		g_warning ("invalid interface: %s\n", args[1]);
+		return;
+	}
+
 	skfd = iw_sockets_open ();
 	if (skfd < 0)
 		return;
@@ -1004,6 +1250,13 @@ netdaemon_init_hash (void)
 	g_hash_table_insert (hash, "accesspoints",
 			     netdaemon_do_get_accesspoints);
 	g_hash_table_insert (hash, "essid", netdaemon_do_change_essid);
+
+#if 0
+	g_hash_table_insert (hash, "setup_wap", netdaemon_do_setup_wap);
+	g_hash_table_insert (hash, "share_interface", netdaemon_do_share_interface);
+	g_hash_table_insert (hash, "start_forward", netdaemon_do_start_forward);
+	g_hash_table_insert (hash, "stop_forward", netdaemon_do_stop_forward);
+#endif
 }
 
 static gboolean
